/* Header for module citinfomod, generated by p2c */
#ifndef CITINFOMOD_H
#define CITINFOMOD_H



#ifndef SYSGLOBALS_H
#include <p2c/sysglobals.h>
#endif



#ifdef CITINFOMOD_G
# define vextern
#else
# define vextern extern
#endif



#define P_citStandAlone 0
#define P_citCS10Land   1
#define P_citCES        2
#define P_citWatson     3



typedef struct P_citinforec {
    Char username[31];
    Char homepath[fidleng + 1];
    datetimerec loginat;
    short node, userid, groupid;
    Char machine[20];
    boolean novice;
} P_citinforec;



vextern P_citinforec *P_citinfo;
vextern _PROCEDURE P_changeCitInfoHook;
vextern long P_citSystem;



#undef vextern

#endif /*CITINFOMOD_H*/

/* End. */

