/* Header for module lunix_pas, generated by p2c */
#ifndef LUNIX_PAS_H
#define LUNIX_PAS_H



#ifndef SYSGLOBALS_H
#include <p2c/sysglobals.h>
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif



#ifdef LUNIX_PAS_G
# define vextern
#else
# define vextern extern
#endif



/* CAGED_date="r      lunix_date0='lunix of $X'; { $u $]" */
#define lunix_lunix_date0 "lunix of Nov 2, 1988 11:45 pm"



/* dave */



#define lunix_lunix_date lunix_lunix_date0

vextern Const datetimerec lunix_null_dt;



/* utmp/wtmp/.times stuff removed */

typedef newci_argarr lunix_lunix_sarr;



/* since 1970 */
/* add to seconds */

typedef struct lunix_fasttimerec {
    long seconds;
    short centisec;
} lunix_fasttimerec;



/*originally from lunix_asm*/



extern boolean lunix_willprompt PV( );
extern Void lunix_getargs PP((Char *p, _PROCEDURE main));
extern Void lunix_getEscErrMsg PP((Char *s, long e));
extern Void lunix_fperror PP((FILE *out, Char *s));
extern Void lunix_perror PP((Char *s));
extern Void lunix_xbomb PP((Char *s, int esc));
extern Void lunix_bomb PP((Char *s));
extern Void lunix_setlog PP((Char *fname));
extern Void lunix_dolog PP((Char *s));
extern Void lunix_dologerr PP((Char *s));
extern Char *lunix_spoolname PP((Char *lunix_Result, Char *directory,
				 Char *extension));
extern long lunix_datetoint PP((daterec date, timerec time));
extern Void lunix_intToDate (long idate, daterec *date, timerec *time);
extern Void lunix_fasttime PP((lunix_fasttimerec *f));
extern Void lunix_datetostr PP((datetimerec datetime, Char *s));
extern Void lunix_qsort PP((Char *a, int n, int es, _PROCEDURE fc));
extern Void lunix_xfixname PP((Char *name, Char *ext));
extern Void lunix_moreln PV( );
extern Void lunix_morestart PV( );
extern Void lunix_moreend PV( );
extern Char *lunix_depasswd PP((Char *lunix_Result, Char *s));
extern Char *lunix_hexout PP((Char *lunix_Result, long value, long size));



#undef vextern

#endif /*LUNIX_PAS_H*/

/* End. */

