/* Header for module misc, generated by p2c */
#ifndef MISC_H
#define MISC_H



/*homeless orphans*/
#ifndef SYSGLOBALS_H
#include <p2c/sysglobals.h>
#endif

#ifndef ASM_H
#include <p2c/asm.h>
#endif



#ifdef MISC_G
# define vextern
#else
# define vextern extern
#endif



#define misc_null       0

#define misc_nullchar   '\0'
#define misc_homechar   '\001'

#define misc_etx        3
#define misc_bell       7

#define misc_bellchar   '\007'

#define misc_bs         8

#define misc_leftchar   '\b'

#define misc_tab        9

#define misc_cteol      '\t'

#define misc_lf         10

#define misc_downchar   '\n'

#define misc_vt         11

#define misc_cteos      '\013'

#define misc_ff         12

#define misc_clearscr   '\f'

#define misc_cr         13

#define misc_eol        '\015'

#define misc_dle        16
#define misc_esc        27

#define misc_escchar    '\033'

#define misc_fsp        28

#define misc_rightchar  '\034'

#define misc_us         31

#define misc_upchar     '\037'

#define misc_del        127
#define misc_cntrl      255



/* CATALOGUE INFORMATION, zero entry refers to the directory itself */
/*name of file or directory*/
/*external file type (LIF)*/
/*file kind*/
/*physical size of file  */
/*                                         or of total data space on volume*/
/*logical size of file  */
/*                                         or unused space on medium*/
/*starting location of file  */
/*                                         or first possible data location*/
/*size of a sector or block*/
/*creation, last modified dates*/
/*creation, last modified times*/
/*extension  */
/*                                         or total possible number of files */
/*                                         or requested number of files*/
/*secondary discretionary field */
/*                                         or start index of requested catalog*/
/*comment or miscellaneous information*/

typedef struct misc_catentry {
    Char cname[tidleng + 1];
    short ceft;
    filekind ckind;
    long cpsize, clsize, cstart, cblocksize;
    daterec ccreatedate, clastdate;
    timerec ccreatetime, clasttime;
    long cextra1, cextra2;
    Char cinfo[21];
} misc_catentry;

typedef struct misc_passentry {
    long pbits;
    Char pword[passleng + 1];
} misc_passentry;



vextern uchar misc_idle;



/* idle character -- 3.0 bug jws 3/20/84 */
extern Void misc_getioerrmsg PP((Char *s, long lastior));
extern Void misc_printerror PP((long errorcode, long lastior));
extern Void misc_upc PP((Char *s));
extern long misc_ueovbytes PP((int unit));
extern Void misc_unblockeddam PP((fib *f, int unum, int request));
extern Void misc_initfilekinds PV();
extern Void misc_lockup PV();
extern Void misc_lockdown PV();

extern Char *tilde_expand PP( (Char *s) );


#undef vextern

#endif /*MISC_H*/

/* End. */

