/* Header for module hpib_0, generated by p2c */
#ifndef HPIB_0_H
#define HPIB_0_H



/* by      Tim Mikkelsen */
/*           date    07/17/81 */
/*           update  09/17/81 */
/*            */
/*           purpose This module contains the LEVEL 0 HPIB GROUP procedures.    */
/*                  */
/*         */

#ifndef IODECLARATIONS_H
#include <p2c/iodecl.h>
#endif



#ifdef HPIB_0_G
# define vextern
#else
# define vextern extern
#endif
extern Void set_hpib PP((int select_code, int line));
extern Void clear_hpib PP((int select_code, int line));
extern boolean hpib_line PP((int select_code, int line));



#undef vextern

#endif /*HPIB_0_H*/

/* End. */

