/* Header for module hpib_1, generated by p2c */
#ifndef HPIB_1_H
#define HPIB_1_H



/* by      Tim Mikkelsen */
/*           date    07/16/81 */
/*           update  08/03/83 by J Schmidt */
/*            */
/*           purpose This module contains the LEVEL 1 HPIB GROUP procedures.    */
/*                  */
/*         */

#ifndef IODECLARATIONS_H
#include <p2c/iodecl.h>
#endif



#ifdef HPIB_1_G
# define vextern
#else
# define vextern extern
#endif
extern Void send_command PP((int select_code, int command));
extern char my_address PP((int select_code));
extern boolean active_controller PP((int select_code));
extern boolean system_controller PP((int select_code));
extern char addr_to_talk PP((long device));
extern char addr_to_listen PP((long device));
extern char set_to_talk PP((long device));
extern char set_to_listen PP((long device));
extern boolean end_set PP((int select_code));



#undef vextern

#endif /*HPIB_1_H*/

/* End. */

