/* Header for module hpib_2, generated by p2c */
#ifndef HPIB_2_H
#define HPIB_2_H



/* by      Tim Mikkelsen */
/*           date    07/17/81 */
/*           update  03/09/83 */
/*            */
/*           purpose This module contains the LEVEL  */
/*                   2 HPIB GROUP procedures.    */
/*                  */
/*         */

#ifndef IODECLARATIONS_H
#include <p2c/iodecl.h>
#endif



#ifdef HPIB_2_G
# define vextern
#else
# define vextern extern
#endif
extern Void abort_hpib PP((int select_code));
extern Void P_clear PP((long device));
extern Void P_listen PP((int select_code, int address));
extern Void P_local PP((long device));
extern Void local_lockout PP((int select_code));
extern Void pass_control PP((long device));
extern Void ppoll_configure PP((long device, long mask));
extern Void ppoll_unconfigure PP((long device));
extern Void P_remote PP((long device));
extern Void P_secondary PP((int select_code, int address));
extern Void P_talk PP((int select_code, int address));
extern Void P_trigger PP((long device));
extern Void P_unlisten PP((int select_code));
extern Void P_untalk PP((int select_code));



#undef vextern

#endif /*HPIB_2_H*/

/* End. */

