/* Header for module GENERAL_3, generated by p2c */
#ifndef GENERAL_3_H
#define GENERAL_3_H



#ifndef IODECLARATIONS_H
#include <p2c/iodecl.h>
#endif



#ifdef GENERAL_3_G
# define vextern
#else
# define vextern extern
#endif
extern Char *ioerror_message PP((Char *P_Result, long ioerror));



#undef vextern

#endif /*GENERAL_3_H*/

/* End. */

